#!/usr/bin/python

from troposphere import Output, Parameter, Ref, Template
from troposphere.dynamodb import (
    AttributeDefinition,
    GlobalSecondaryIndex,
    KeySchema,
    Projection,
    ProvisionedThroughput,
    Table,
)

template = Template()

template.set_description("Create a dynamodb table with a global secondary " "index")
# N.B. If you remove the provisioning section this works for
# LocalSecondaryIndexes aswell.

readunits = template.add_parameter(
    Parameter(
        "ReadCapacityUnits",
        Description="Provisioned read throughput",
        Type="Number",
        Default="10",
        MinValue="5",
        MaxValue="10000",
        ConstraintDescription="should be between 5 and 10000",
    )
)

writeunits = template.add_parameter(
    Parameter(
        "WriteCapacityUnits",
        Description="Provisioned write throughput",
        Type="Number",
        Default="5",
        MinValue="5",
        MaxValue="10000",
        ConstraintDescription="should be between 5 and 10000",
    )
)

tableIndexName = template.add_parameter(
    Parameter(
        "TableIndexName",
        Description="Table: Primary Key Field",
        Type="String",
        Default="id",
        AllowedPattern="[a-zA-Z0-9]*",
        MinLength="1",
        MaxLength="2048",
        ConstraintDescription="must contain only alphanumberic characters",
    )
)

tableIndexDataType = template.add_parameter(
    Parameter(
        "TableIndexDataType",
        Description=" Table: Primary Key Data Type",
        Type="String",
        Default="S",
        AllowedPattern="[S|N|B]",
        MinLength="1",
        MaxLength="1",
        ConstraintDescription="S for string data, N for numeric data, or B for "
        "binary data",
    )
)

secondaryIndexHashName = template.add_parameter(
    Parameter(
        "SecondaryIndexHashName",
        Description="Secondary Index: Primary Key Field",
        Type="String",
        Default="tokenType",
        AllowedPattern="[a-zA-Z0-9]*",
        MinLength="1",
        MaxLength="2048",
        ConstraintDescription="must contain only alphanumberic characters",
    )
)

secondaryIndexHashDataType = template.add_parameter(
    Parameter(
        "SecondaryIndexHashDataType",
        Description="Secondary Index: Primary Key Data Type",
        Type="String",
        Default="S",
        AllowedPattern="[S|N|B]",
        MinLength="1",
        MaxLength="1",
        ConstraintDescription="S for string data, N for numeric data, or B for "
        "binary data",
    )
)

secondaryIndexRangeName = template.add_parameter(
    Parameter(
        "refreshSecondaryIndexRangeName",
        Description="Secondary Index: Range Key Field",
        Type="String",
        Default="tokenUpdatedTime",
        AllowedPattern="[a-zA-Z0-9]*",
        MinLength="1",
        MaxLength="2048",
        ConstraintDescription="must contain only alphanumberic characters",
    )
)

secondaryIndexRangeDataType = template.add_parameter(
    Parameter(
        "SecondaryIndexRangeDataType",
        Description="Secondary Index: Range Key Data Type",
        Type="String",
        Default="S",
        AllowedPattern="[S|N|B]",
        MinLength="1",
        MaxLength="1",
        ConstraintDescription="S for string data, N for numeric data, or B for "
        "binary data",
    )
)


GSITable = template.add_resource(
    Table(
        "GSITable",
        AttributeDefinitions=[
            AttributeDefinition(
                AttributeName=Ref(tableIndexName), AttributeType=Ref(tableIndexDataType)
            ),
            AttributeDefinition(
                AttributeName=Ref(secondaryIndexHashName),
                AttributeType=Ref(secondaryIndexHashDataType),
            ),
            AttributeDefinition(
                AttributeName=Ref(secondaryIndexRangeName),
                AttributeType=Ref(secondaryIndexRangeDataType),
            ),
            AttributeDefinition(AttributeName="non_key_attribute_0", AttributeType="S"),
            AttributeDefinition(AttributeName="non_key_attribute_1", AttributeType="S"),
        ],
        KeySchema=[KeySchema(AttributeName=Ref(tableIndexName), KeyType="HASH")],
        ProvisionedThroughput=ProvisionedThroughput(
            ReadCapacityUnits=Ref(readunits), WriteCapacityUnits=Ref(writeunits)
        ),
        GlobalSecondaryIndexes=[
            GlobalSecondaryIndex(
                IndexName="SecondaryIndex",
                KeySchema=[
                    KeySchema(
                        AttributeName=Ref(secondaryIndexHashName), KeyType="HASH"
                    ),
                    KeySchema(
                        AttributeName=Ref(secondaryIndexRangeName), KeyType="RANGE"
                    ),
                ],
                Projection=Projection(
                    ProjectionType="INCLUDE", NonKeyAttributes=["non_key_attribute_0"]
                ),
                ProvisionedThroughput=ProvisionedThroughput(
                    ReadCapacityUnits=Ref(readunits), WriteCapacityUnits=Ref(writeunits)
                ),
            )
        ],
    )
)

template.add_output(
    Output(
        "GSITable",
        Value=Ref(GSITable),
        Description="Table with a Global Secondary Index",
    )
)


print(template.to_json())



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.


from . import json_checker, s3_bucket_name


def validate_document_content(x):
    """
    Property: Document.Content
    """

    def check_json(x):
        import json

        try:
            json.loads(x)
            return True
        except json.decoder.JSONDecodeError:
            return False

    def check_yaml(x):
        import yaml

        try:
            yaml.safe_load(x)
            return True
        except yaml.composer.ComposerError:
            return False

    if isinstance(x, dict):
        return x

    if check_json(x) or check_yaml(x):
        return x

    raise ValueError("Content must be one of dict or json/yaml string")


def validate_json_checker(x):
    """
    Property: MaintenanceWindowLambdaParameters.Payload
    """
    return json_checker(x)


def compliance_level(level):
    """
    Property: PatchBaseline.ApprovedPatchesComplianceLevel
    Property: Rule.ComplianceLevel
    """

    valid_levels = ["CRITICAL", "HIGH", "MEDIUM", "LOW", "INFORMATIONAL", "UNSPECIFIED"]
    if level not in valid_levels:
        raise ValueError(
            'ApprovedPatchesComplianceLevel must be one of: "%s"'
            % (", ".join(valid_levels))
        )
    return level


def notification_event(events):
    """
    Property: NotificationConfig.NotificationEvents
    """

    valid_events = ["All", "InProgress", "Success", "TimedOut", "Cancelled", "Failed"]
    for event in events:
        if event not in valid_events:
            raise ValueError(
                'NotificationEvents must be at least one of: "%s"'
                % (", ".join(valid_events))
            )
    return events


def notification_type(notification):
    """
    Property: NotificationConfig.NotificationType
    """

    valid_notifications = ["Command", "Invocation"]
    if notification not in valid_notifications:
        raise ValueError(
            'NotificationType must be one of: "%s"' % (", ".join(valid_notifications))
        )
    return notification


def operating_system(os):
    """
    Property: PatchBaseline.OperatingSystem
    """

    valid_os = [
        "AMAZON_LINUX",
        "AMAZON_LINUX_2",
        "CENTOS",
        "DEBIAN",
        "MACOS",
        "ORACLE_LINUX",
        "RASPBIAN",
        "REDHAT_ENTERPRISE_LINUX",
        "ROCKY_LINUX",
        "SUSE",
        "UBUNTU",
        "WINDOWS",
    ]
    if os not in valid_os:
        raise ValueError('OperatingSystem must be one of: "%s"' % (", ".join(valid_os)))
    return os


def validate_s3_bucket_name(b):
    """
    Property: LoggingInfo.S3Bucket
    Property: MaintenanceWindowRunCommandParameters.OutputS3BucketName
    """

    return s3_bucket_name(b)


def task_type(task):
    """
    Property: MaintenanceWindowTask.TaskType
    """

    valid_tasks = ["RUN_COMMAND", "AUTOMATION", "LAMBDA", "STEP_FUNCTION"]
    if task not in valid_tasks:
        raise ValueError('TaskType must be one of: "%s"' % (", ".join(valid_tasks)))
    return task



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***


from . import AWSObject, AWSProperty, PropsDictType, Tags
from .validators import boolean, integer


class AutoStartConfiguration(AWSProperty):
    """
    `AutoStartConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emrserverless-application-autostartconfiguration.html>`__
    """

    props: PropsDictType = {
        "Enabled": (boolean, False),
    }


class AutoStopConfiguration(AWSProperty):
    """
    `AutoStopConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emrserverless-application-autostopconfiguration.html>`__
    """

    props: PropsDictType = {
        "Enabled": (boolean, False),
        "IdleTimeoutMinutes": (integer, False),
    }


class ConfigurationObject(AWSProperty):
    """
    `ConfigurationObject <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emrserverless-application-configurationobject.html>`__
    """

    props: PropsDictType = {
        "Classification": (str, True),
        "Properties": (dict, False),
    }


class ImageConfigurationInput(AWSProperty):
    """
    `ImageConfigurationInput <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emrserverless-application-imageconfigurationinput.html>`__
    """

    props: PropsDictType = {
        "ImageUri": (str, False),
    }


class WorkerConfiguration(AWSProperty):
    """
    `WorkerConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emrserverless-application-workerconfiguration.html>`__
    """

    props: PropsDictType = {
        "Cpu": (str, True),
        "Disk": (str, False),
        "Memory": (str, True),
    }


class InitialCapacityConfig(AWSProperty):
    """
    `InitialCapacityConfig <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emrserverless-application-initialcapacityconfig.html>`__
    """

    props: PropsDictType = {
        "WorkerConfiguration": (WorkerConfiguration, True),
        "WorkerCount": (integer, True),
    }


class InitialCapacityConfigKeyValuePair(AWSProperty):
    """
    `InitialCapacityConfigKeyValuePair <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emrserverless-application-initialcapacityconfigkeyvaluepair.html>`__
    """

    props: PropsDictType = {
        "Key": (str, True),
        "Value": (InitialCapacityConfig, True),
    }


class MaximumAllowedResources(AWSProperty):
    """
    `MaximumAllowedResources <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emrserverless-application-maximumallowedresources.html>`__
    """

    props: PropsDictType = {
        "Cpu": (str, True),
        "Disk": (str, False),
        "Memory": (str, True),
    }


class LogTypeMapKeyValuePair(AWSProperty):
    """
    `LogTypeMapKeyValuePair <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emrserverless-application-logtypemapkeyvaluepair.html>`__
    """

    props: PropsDictType = {
        "Key": (str, True),
        "Value": ([str], True),
    }


class CloudWatchLoggingConfiguration(AWSProperty):
    """
    `CloudWatchLoggingConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emrserverless-application-cloudwatchloggingconfiguration.html>`__
    """

    props: PropsDictType = {
        "Enabled": (boolean, False),
        "EncryptionKeyArn": (str, False),
        "LogGroupName": (str, False),
        "LogStreamNamePrefix": (str, False),
        "LogTypeMap": ([LogTypeMapKeyValuePair], False),
    }


class ManagedPersistenceMonitoringConfiguration(AWSProperty):
    """
    `ManagedPersistenceMonitoringConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emrserverless-application-managedpersistencemonitoringconfiguration.html>`__
    """

    props: PropsDictType = {
        "Enabled": (boolean, False),
        "EncryptionKeyArn": (str, False),
    }


class S3MonitoringConfiguration(AWSProperty):
    """
    `S3MonitoringConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emrserverless-application-s3monitoringconfiguration.html>`__
    """

    props: PropsDictType = {
        "EncryptionKeyArn": (str, False),
        "LogUri": (str, False),
    }


class MonitoringConfiguration(AWSProperty):
    """
    `MonitoringConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emrserverless-application-monitoringconfiguration.html>`__
    """

    props: PropsDictType = {
        "CloudWatchLoggingConfiguration": (CloudWatchLoggingConfiguration, False),
        "ManagedPersistenceMonitoringConfiguration": (
            ManagedPersistenceMonitoringConfiguration,
            False,
        ),
        "S3MonitoringConfiguration": (S3MonitoringConfiguration, False),
    }


class NetworkConfiguration(AWSProperty):
    """
    `NetworkConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emrserverless-application-networkconfiguration.html>`__
    """

    props: PropsDictType = {
        "SecurityGroupIds": ([str], False),
        "SubnetIds": ([str], False),
    }


class WorkerTypeSpecificationInput(AWSProperty):
    """
    `WorkerTypeSpecificationInput <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emrserverless-application-workertypespecificationinput.html>`__
    """

    props: PropsDictType = {
        "ImageConfiguration": (ImageConfigurationInput, False),
    }


class Application(AWSObject):
    """
    `Application <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-emrserverless-application.html>`__
    """

    resource_type = "AWS::EMRServerless::Application"

    props: PropsDictType = {
        "Architecture": (str, False),
        "AutoStartConfiguration": (AutoStartConfiguration, False),
        "AutoStopConfiguration": (AutoStopConfiguration, False),
        "ImageConfiguration": (ImageConfigurationInput, False),
        "InitialCapacity": ([InitialCapacityConfigKeyValuePair], False),
        "MaximumCapacity": (MaximumAllowedResources, False),
        "MonitoringConfiguration": (MonitoringConfiguration, False),
        "Name": (str, False),
        "NetworkConfiguration": (NetworkConfiguration, False),
        "ReleaseLabel": (str, True),
        "RuntimeConfiguration": ([ConfigurationObject], False),
        "Tags": (Tags, False),
        "Type": (str, True),
        "WorkerTypeSpecifications": (dict, False),
    }



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***


from . import AWSObject, AWSProperty, PropsDictType
from .validators import boolean, integer


class ModuleLoggingConfiguration(AWSProperty):
    """
    `ModuleLoggingConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-mwaa-environment-moduleloggingconfiguration.html>`__
    """

    props: PropsDictType = {
        "CloudWatchLogGroupArn": (str, False),
        "Enabled": (boolean, False),
        "LogLevel": (str, False),
    }


class LoggingConfiguration(AWSProperty):
    """
    `LoggingConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-mwaa-environment-loggingconfiguration.html>`__
    """

    props: PropsDictType = {
        "DagProcessingLogs": (ModuleLoggingConfiguration, False),
        "SchedulerLogs": (ModuleLoggingConfiguration, False),
        "TaskLogs": (ModuleLoggingConfiguration, False),
        "WebserverLogs": (ModuleLoggingConfiguration, False),
        "WorkerLogs": (ModuleLoggingConfiguration, False),
    }


class NetworkConfiguration(AWSProperty):
    """
    `NetworkConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-mwaa-environment-networkconfiguration.html>`__
    """

    props: PropsDictType = {
        "SecurityGroupIds": ([str], False),
        "SubnetIds": ([str], False),
    }


class Environment(AWSObject):
    """
    `Environment <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-mwaa-environment.html>`__
    """

    resource_type = "AWS::MWAA::Environment"

    props: PropsDictType = {
        "AirflowConfigurationOptions": (dict, False),
        "AirflowVersion": (str, False),
        "DagS3Path": (str, False),
        "EndpointManagement": (str, False),
        "EnvironmentClass": (str, False),
        "ExecutionRoleArn": (str, False),
        "KmsKey": (str, False),
        "LoggingConfiguration": (LoggingConfiguration, False),
        "MaxWorkers": (integer, False),
        "MinWorkers": (integer, False),
        "Name": (str, True),
        "NetworkConfiguration": (NetworkConfiguration, False),
        "PluginsS3ObjectVersion": (str, False),
        "PluginsS3Path": (str, False),
        "RequirementsS3ObjectVersion": (str, False),
        "RequirementsS3Path": (str, False),
        "Schedulers": (integer, False),
        "SourceBucketArn": (str, False),
        "StartupScriptS3ObjectVersion": (str, False),
        "StartupScriptS3Path": (str, False),
        "Tags": (dict, False),
        "WebserverAccessMode": (str, False),
        "WeeklyMaintenanceWindowStart": (str, False),
    }



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***


from . import AWSObject, AWSProperty, PropsDictType, Tags
from .validators import boolean, integer


class UnusedAccessConfiguration(AWSProperty):
    """
    `UnusedAccessConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-accessanalyzer-analyzer-unusedaccessconfiguration.html>`__
    """

    props: PropsDictType = {
        "UnusedAccessAge": (integer, False),
    }


class AnalyzerConfiguration(AWSProperty):
    """
    `AnalyzerConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-accessanalyzer-analyzer-analyzerconfiguration.html>`__
    """

    props: PropsDictType = {
        "UnusedAccessConfiguration": (UnusedAccessConfiguration, False),
    }


class Filter(AWSProperty):
    """
    `Filter <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-accessanalyzer-analyzer-filter.html>`__
    """

    props: PropsDictType = {
        "Contains": ([str], False),
        "Eq": ([str], False),
        "Exists": (boolean, False),
        "Neq": ([str], False),
        "Property": (str, True),
    }


class ArchiveRule(AWSProperty):
    """
    `ArchiveRule <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-accessanalyzer-analyzer-archiverule.html>`__
    """

    props: PropsDictType = {
        "Filter": ([Filter], True),
        "RuleName": (str, True),
    }


class Analyzer(AWSObject):
    """
    `Analyzer <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-accessanalyzer-analyzer.html>`__
    """

    resource_type = "AWS::AccessAnalyzer::Analyzer"

    props: PropsDictType = {
        "AnalyzerConfiguration": (AnalyzerConfiguration, False),
        "AnalyzerName": (str, False),
        "ArchiveRules": ([ArchiveRule], False),
        "Tags": (Tags, False),
        "Type": (str, True),
    }



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***


from . import AWSObject, AWSProperty, PropsDictType, Tags
from .validators import boolean, double, integer


class DBClusterRole(AWSProperty):
    """
    `DBClusterRole <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-neptune-dbcluster-dbclusterrole.html>`__
    """

    props: PropsDictType = {
        "FeatureName": (str, False),
        "RoleArn": (str, True),
    }


class ServerlessScalingConfiguration(AWSProperty):
    """
    `ServerlessScalingConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-neptune-dbcluster-serverlessscalingconfiguration.html>`__
    """

    props: PropsDictType = {
        "MaxCapacity": (double, True),
        "MinCapacity": (double, True),
    }


class DBCluster(AWSObject):
    """
    `DBCluster <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-neptune-dbcluster.html>`__
    """

    resource_type = "AWS::Neptune::DBCluster"

    props: PropsDictType = {
        "AssociatedRoles": ([DBClusterRole], False),
        "AvailabilityZones": ([str], False),
        "BackupRetentionPeriod": (integer, False),
        "CopyTagsToSnapshot": (boolean, False),
        "DBClusterIdentifier": (str, False),
        "DBClusterParameterGroupName": (str, False),
        "DBInstanceParameterGroupName": (str, False),
        "DBPort": (integer, False),
        "DBSubnetGroupName": (str, False),
        "DeletionProtection": (boolean, False),
        "EnableCloudwatchLogsExports": ([str], False),
        "EngineVersion": (str, False),
        "IamAuthEnabled": (boolean, False),
        "KmsKeyId": (str, False),
        "PreferredBackupWindow": (str, False),
        "PreferredMaintenanceWindow": (str, False),
        "RestoreToTime": (str, False),
        "RestoreType": (str, False),
        "ServerlessScalingConfiguration": (ServerlessScalingConfiguration, False),
        "SnapshotIdentifier": (str, False),
        "SourceDBClusterIdentifier": (str, False),
        "StorageEncrypted": (boolean, False),
        "Tags": (Tags, False),
        "UseLatestRestorableTime": (boolean, False),
        "VpcSecurityGroupIds": ([str], False),
    }


class DBClusterParameterGroup(AWSObject):
    """
    `DBClusterParameterGroup <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-neptune-dbclusterparametergroup.html>`__
    """

    resource_type = "AWS::Neptune::DBClusterParameterGroup"

    props: PropsDictType = {
        "Description": (str, True),
        "Family": (str, True),
        "Name": (str, False),
        "Parameters": (dict, True),
        "Tags": (Tags, False),
    }


class DBInstance(AWSObject):
    """
    `DBInstance <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-neptune-dbinstance.html>`__
    """

    resource_type = "AWS::Neptune::DBInstance"

    props: PropsDictType = {
        "AllowMajorVersionUpgrade": (boolean, False),
        "AutoMinorVersionUpgrade": (boolean, False),
        "AvailabilityZone": (str, False),
        "DBClusterIdentifier": (str, False),
        "DBInstanceClass": (str, True),
        "DBInstanceIdentifier": (str, False),
        "DBParameterGroupName": (str, False),
        "DBSnapshotIdentifier": (str, False),
        "DBSubnetGroupName": (str, False),
        "PreferredMaintenanceWindow": (str, False),
        "Tags": (Tags, False),
    }


class DBParameterGroup(AWSObject):
    """
    `DBParameterGroup <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-neptune-dbparametergroup.html>`__
    """

    resource_type = "AWS::Neptune::DBParameterGroup"

    props: PropsDictType = {
        "Description": (str, True),
        "Family": (str, True),
        "Name": (str, False),
        "Parameters": (dict, True),
        "Tags": (Tags, False),
    }


class DBSubnetGroup(AWSObject):
    """
    `DBSubnetGroup <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-neptune-dbsubnetgroup.html>`__
    """

    resource_type = "AWS::Neptune::DBSubnetGroup"

    props: PropsDictType = {
        "DBSubnetGroupDescription": (str, True),
        "DBSubnetGroupName": (str, False),
        "SubnetIds": ([str], True),
        "Tags": (Tags, False),
    }



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***


from . import AWSObject, AWSProperty, PropsDictType, Tags
from .validators import boolean
from .validators.sns import policytypes


class SubscriptionResource(AWSObject):
    """
    `SubscriptionResource <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-sns-subscription.html>`__
    """

    resource_type = "AWS::SNS::Subscription"

    props: PropsDictType = {
        "DeliveryPolicy": (dict, False),
        "Endpoint": (str, False),
        "FilterPolicy": (dict, False),
        "FilterPolicyScope": (str, False),
        "Protocol": (str, True),
        "RawMessageDelivery": (boolean, False),
        "RedrivePolicy": (dict, False),
        "Region": (str, False),
        "ReplayPolicy": (dict, False),
        "SubscriptionRoleArn": (str, False),
        "TopicArn": (str, True),
    }


class LoggingConfig(AWSProperty):
    """
    `LoggingConfig <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-sns-topic-loggingconfig.html>`__
    """

    props: PropsDictType = {
        "FailureFeedbackRoleArn": (str, False),
        "Protocol": (str, True),
        "SuccessFeedbackRoleArn": (str, False),
        "SuccessFeedbackSampleRate": (str, False),
    }


class Subscription(AWSProperty):
    """
    `Subscription <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-sns-topic-subscription.html>`__
    """

    props: PropsDictType = {
        "Endpoint": (str, True),
        "Protocol": (str, True),
    }


class Topic(AWSObject):
    """
    `Topic <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-sns-topic.html>`__
    """

    resource_type = "AWS::SNS::Topic"

    props: PropsDictType = {
        "ArchivePolicy": (dict, False),
        "ContentBasedDeduplication": (boolean, False),
        "DataProtectionPolicy": (dict, False),
        "DeliveryStatusLogging": ([LoggingConfig], False),
        "DisplayName": (str, False),
        "FifoTopic": (boolean, False),
        "KmsMasterKeyId": (str, False),
        "SignatureVersion": (str, False),
        "Subscription": ([Subscription], False),
        "Tags": (Tags, False),
        "TopicName": (str, False),
        "TracingConfig": (str, False),
    }


class TopicInlinePolicy(AWSObject):
    """
    `TopicInlinePolicy <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-sns-topicinlinepolicy.html>`__
    """

    resource_type = "AWS::SNS::TopicInlinePolicy"

    props: PropsDictType = {
        "PolicyDocument": (dict, True),
        "TopicArn": (str, True),
    }


class TopicPolicy(AWSObject):
    """
    `TopicPolicy <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-sns-topicpolicy.html>`__
    """

    resource_type = "AWS::SNS::TopicPolicy"

    props: PropsDictType = {
        "PolicyDocument": (policytypes, True),
        "Topics": ([str], True),
    }



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***


from . import AWSObject, AWSProperty, PropsDictType, Tags
from .validators import boolean, integer


class PipeEnrichmentHttpParameters(AWSProperty):
    """
    `PipeEnrichmentHttpParameters <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pipes-pipe-pipeenrichmenthttpparameters.html>`__
    """

    props: PropsDictType = {
        "HeaderParameters": (dict, False),
        "PathParameterValues": ([str], False),
        "QueryStringParameters": (dict, False),
    }


class PipeEnrichmentParameters(AWSProperty):
    """
    `PipeEnrichmentParameters <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pipes-pipe-pipeenrichmentparameters.html>`__
    """

    props: PropsDictType = {
        "HttpParameters": (PipeEnrichmentHttpParameters, False),
        "InputTemplate": (str, False),
    }


class CloudwatchLogsLogDestination(AWSProperty):
    """
    `CloudwatchLogsLogDestination <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pipes-pipe-cloudwatchlogslogdestination.html>`__
    """

    props: PropsDictType = {
        "LogGroupArn": (str, False),
    }


class FirehoseLogDestination(AWSProperty):
    """
    `FirehoseLogDestination <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pipes-pipe-firehoselogdestination.html>`__
    """

    props: PropsDictType = {
        "DeliveryStreamArn": (str, False),
    }


class S3LogDestination(AWSProperty):
    """
    `S3LogDestination <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pipes-pipe-s3logdestination.html>`__
    """

    props: PropsDictType = {
        "BucketName": (str, False),
        "BucketOwner": (str, False),
        "OutputFormat": (str, False),
        "Prefix": (str, False),
    }


class PipeLogConfiguration(AWSProperty):
    """
    `PipeLogConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pipes-pipe-pipelogconfiguration.html>`__
    """

    props: PropsDictType = {
        "CloudwatchLogsLogDestination": (CloudwatchLogsLogDestination, False),
        "FirehoseLogDestination": (FirehoseLogDestination, False),
        "IncludeExecutionData": ([str], False),
        "Level": (str, False),
        "S3LogDestination": (S3LogDestination, False),
    }


class Filter(AWSProperty):
    """
    `Filter <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pipes-pipe-filter.html>`__
    """

    props: PropsDictType = {
        "Pattern": (str, False),
    }


class FilterCriteria(AWSProperty):
    """
    `FilterCriteria <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pipes-pipe-filtercriteria.html>`__
    """

    props: PropsDictType = {
        "Filters": ([Filter], False),
    }


class MQBrokerAccessCredentials(AWSProperty):
    """
    `MQBrokerAccessCredentials <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pipes-pipe-mqbrokeraccesscredentials.html>`__
    """

    props: PropsDictType = {
        "BasicAuth": (str, True),
    }


class PipeSourceActiveMQBrokerParameters(AWSProperty):
    """
    `PipeSourceActiveMQBrokerParameters <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pipes-pipe-pipesourceactivemqbrokerparameters.html>`__
    """

    props: PropsDictType = {
        "BatchSize": (integer, False),
        "Credentials": (MQBrokerAccessCredentials, True),
        "MaximumBatchingWindowInSeconds": (integer, False),
        "QueueName": (str, True),
    }


class DeadLetterConfig(AWSProperty):
    """
    `DeadLetterConfig <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pipes-pipe-deadletterconfig.html>`__
    """

    props: PropsDictType = {
        "Arn": (str, False),
    }


class PipeSourceDynamoDBStreamParameters(AWSProperty):
    """
    `PipeSourceDynamoDBStreamParameters <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pipes-pipe-pipesourcedynamodbstreamparameters.html>`__
    """

    props: PropsDictType = {
        "BatchSize": (integer, False),
        "DeadLetterConfig": (DeadLetterConfig, False),
        "MaximumBatchingWindowInSeconds": (integer, False),
        "MaximumRecordAgeInSeconds": (integer, False),
        "MaximumRetryAttempts": (integer, False),
        "OnPartialBatchItemFailure": (str, False),
        "ParallelizationFactor": (integer, False),
        "StartingPosition": (str, True),
    }


class PipeSourceKinesisStreamParameters(AWSProperty):
    """
    `PipeSourceKinesisStreamParameters <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pipes-pipe-pipesourcekinesisstreamparameters.html>`__
    """

    props: PropsDictType = {
        "BatchSize": (integer, False),
        "DeadLetterConfig": (DeadLetterConfig, False),
        "MaximumBatchingWindowInSeconds": (integer, False),
        "MaximumRecordAgeInSeconds": (integer, False),
        "MaximumRetryAttempts": (integer, False),
        "OnPartialBatchItemFailure": (str, False),
        "ParallelizationFactor": (integer, False),
        "StartingPosition": (str, True),
        "StartingPositionTimestamp": (str, False),
    }


class MSKAccessCredentials(AWSProperty):
    """
    `MSKAccessCredentials <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pipes-pipe-mskaccesscredentials.html>`__
    """

    props: PropsDictType = {
        "ClientCertificateTlsAuth": (str, False),
        "SaslScram512Auth": (str, False),
    }


class PipeSourceManagedStreamingKafkaParameters(AWSProperty):
    """
    `PipeSourceManagedStreamingKafkaParameters <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pipes-pipe-pipesourcemanagedstreamingkafkaparameters.html>`__
    """

    props: PropsDictType = {
        "BatchSize": (integer, False),
        "ConsumerGroupID": (str, False),
        "Credentials": (MSKAccessCredentials, False),
        "MaximumBatchingWindowInSeconds": (integer, False),
        "StartingPosition": (str, False),
        "TopicName": (str, True),
    }


class PipeSourceRabbitMQBrokerParameters(AWSProperty):
    """
    `PipeSourceRabbitMQBrokerParameters <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pipes-pipe-pipesourcerabbitmqbrokerparameters.html>`__
    """

    props: PropsDictType = {
        "BatchSize": (integer, False),
        "Credentials": (MQBrokerAccessCredentials, True),
        "MaximumBatchingWindowInSeconds": (integer, False),
        "QueueName": (str, True),
        "VirtualHost": (str, False),
    }


class SelfManagedKafkaAccessConfigurationCredentials(AWSProperty):
    """
    `SelfManagedKafkaAccessConfigurationCredentials <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pipes-pipe-selfmanagedkafkaaccessconfigurationcredentials.html>`__
    """

    props: PropsDictType = {
        "BasicAuth": (str, False),
        "ClientCertificateTlsAuth": (str, False),
        "SaslScram256Auth": (str, False),
        "SaslScram512Auth": (str, False),
    }


class SelfManagedKafkaAccessConfigurationVpc(AWSProperty):
    """
    `SelfManagedKafkaAccessConfigurationVpc <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pipes-pipe-selfmanagedkafkaaccessconfigurationvpc.html>`__
    """

    props: PropsDictType = {
        "SecurityGroup": ([str], False),
        "Subnets": ([str], False),
    }


class PipeSourceSelfManagedKafkaParameters(AWSProperty):
    """
    `PipeSourceSelfManagedKafkaParameters <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pipes-pipe-pipesourceselfmanagedkafkaparameters.html>`__
    """

    props: PropsDictType = {
        "AdditionalBootstrapServers": ([str], False),
        "BatchSize": (integer, False),
        "ConsumerGroupID": (str, False),
        "Credentials": (SelfManagedKafkaAccessConfigurationCredentials, False),
        "MaximumBatchingWindowInSeconds": (integer, False),
        "ServerRootCaCertificate": (str, False),
        "StartingPosition": (str, False),
        "TopicName": (str, True),
        "Vpc": (SelfManagedKafkaAccessConfigurationVpc, False),
    }


class PipeSourceSqsQueueParameters(AWSProperty):
    """
    `PipeSourceSqsQueueParameters <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pipes-pipe-pipesourcesqsqueueparameters.html>`__
    """

    props: PropsDictType = {
        "BatchSize": (integer, False),
        "MaximumBatchingWindowInSeconds": (integer, False),
    }


class PipeSourceParameters(AWSProperty):
    """
    `PipeSourceParameters <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pipes-pipe-pipesourceparameters.html>`__
    """

    props: PropsDictType = {
        "ActiveMQBrokerParameters": (PipeSourceActiveMQBrokerParameters, False),
        "DynamoDBStreamParameters": (PipeSourceDynamoDBStreamParameters, False),
        "FilterCriteria": (FilterCriteria, False),
        "KinesisStreamParameters": (PipeSourceKinesisStreamParameters, False),
        "ManagedStreamingKafkaParameters": (
            PipeSourceManagedStreamingKafkaParameters,
            False,
        ),
        "RabbitMQBrokerParameters": (PipeSourceRabbitMQBrokerParameters, False),
        "SelfManagedKafkaParameters": (PipeSourceSelfManagedKafkaParameters, False),
        "SqsQueueParameters": (PipeSourceSqsQueueParameters, False),
    }


class BatchArrayProperties(AWSProperty):
    """
    `BatchArrayProperties <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pipes-pipe-batcharrayproperties.html>`__
    """

    props: PropsDictType = {
        "Size": (integer, False),
    }


class BatchEnvironmentVariable(AWSProperty):
    """
    `BatchEnvironmentVariable <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pipes-pipe-batchenvironmentvariable.html>`__
    """

    props: PropsDictType = {
        "Name": (str, False),
        "Value": (str, False),
    }


class BatchResourceRequirement(AWSProperty):
    """
    `BatchResourceRequirement <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pipes-pipe-batchresourcerequirement.html>`__
    """

    props: PropsDictType = {
        "Type": (str, True),
        "Value": (str, True),
    }


class BatchContainerOverrides(AWSProperty):
    """
    `BatchContainerOverrides <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pipes-pipe-batchcontaineroverrides.html>`__
    """

    props: PropsDictType = {
        "Command": ([str], False),
        "Environment": ([BatchEnvironmentVariable], False),
        "InstanceType": (str, False),
        "ResourceRequirements": ([BatchResourceRequirement], False),
    }


class BatchJobDependency(AWSProperty):
    """
    `BatchJobDependency <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pipes-pipe-batchjobdependency.html>`__
    """

    props: PropsDictType = {
        "JobId": (str, False),
        "Type": (str, False),
    }


class BatchRetryStrategy(AWSProperty):
    """
    `BatchRetryStrategy <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pipes-pipe-batchretrystrategy.html>`__
    """

    props: PropsDictType = {
        "Attempts": (integer, False),
    }


class PipeTargetBatchJobParameters(AWSProperty):
    """
    `PipeTargetBatchJobParameters <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pipes-pipe-pipetargetbatchjobparameters.html>`__
    """

    props: PropsDictType = {
        "ArrayProperties": (BatchArrayProperties, False),
        "ContainerOverrides": (BatchContainerOverrides, False),
        "DependsOn": ([BatchJobDependency], False),
        "JobDefinition": (str, True),
        "JobName": (str, True),
        "Parameters": (dict, False),
        "RetryStrategy": (BatchRetryStrategy, False),
    }


class PipeTargetCloudWatchLogsParameters(AWSProperty):
    """
    `PipeTargetCloudWatchLogsParameters <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pipes-pipe-pipetargetcloudwatchlogsparameters.html>`__
    """

    props: PropsDictType = {
        "LogStreamName": (str, False),
        "Timestamp": (str, False),
    }


class CapacityProviderStrategyItem(AWSProperty):
    """
    `CapacityProviderStrategyItem <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pipes-pipe-capacityproviderstrategyitem.html>`__
    """

    props: PropsDictType = {
        "Base": (integer, False),
        "CapacityProvider": (str, True),
        "Weight": (integer, False),
    }


class EcsEnvironmentFile(AWSProperty):
    """
    `EcsEnvironmentFile <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pipes-pipe-ecsenvironmentfile.html>`__
    """

    props: PropsDictType = {
        "Type": (str, True),
        "Value": (str, True),
    }


class EcsEnvironmentVariable(AWSProperty):
    """
    `EcsEnvironmentVariable <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pipes-pipe-ecsenvironmentvariable.html>`__
    """

    props: PropsDictType = {
        "Name": (str, False),
        "Value": (str, False),
    }


class EcsResourceRequirement(AWSProperty):
    """
    `EcsResourceRequirement <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pipes-pipe-ecsresourcerequirement.html>`__
    """

    props: PropsDictType = {
        "Type": (str, True),
        "Value": (str, True),
    }


class EcsContainerOverride(AWSProperty):
    """
    `EcsContainerOverride <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pipes-pipe-ecscontaineroverride.html>`__
    """

    props: PropsDictType = {
        "Command": ([str], False),
        "Cpu": (integer, False),
        "Environment": ([EcsEnvironmentVariable], False),
        "EnvironmentFiles": ([EcsEnvironmentFile], False),
        "Memory": (integer, False),
        "MemoryReservation": (integer, False),
        "Name": (str, False),
        "ResourceRequirements": ([EcsResourceRequirement], False),
    }


class EcsEphemeralStorage(AWSProperty):
    """
    `EcsEphemeralStorage <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pipes-pipe-ecsephemeralstorage.html>`__
    """

    props: PropsDictType = {
        "SizeInGiB": (integer, True),
    }


class EcsInferenceAcceleratorOverride(AWSProperty):
    """
    `EcsInferenceAcceleratorOverride <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pipes-pipe-ecsinferenceacceleratoroverride.html>`__
    """

    props: PropsDictType = {
        "DeviceName": (str, False),
        "DeviceType": (str, False),
    }


class EcsTaskOverride(AWSProperty):
    """
    `EcsTaskOverride <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pipes-pipe-ecstaskoverride.html>`__
    """

    props: PropsDictType = {
        "ContainerOverrides": ([EcsContainerOverride], False),
        "Cpu": (str, False),
        "EphemeralStorage": (EcsEphemeralStorage, False),
        "ExecutionRoleArn": (str, False),
        "InferenceAcceleratorOverrides": ([EcsInferenceAcceleratorOverride], False),
        "Memory": (str, False),
        "TaskRoleArn": (str, False),
    }


class AwsVpcConfiguration(AWSProperty):
    """
    `AwsVpcConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pipes-pipe-awsvpcconfiguration.html>`__
    """

    props: PropsDictType = {
        "AssignPublicIp": (str, False),
        "SecurityGroups": ([str], False),
        "Subnets": ([str], True),
    }


class NetworkConfiguration(AWSProperty):
    """
    `NetworkConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pipes-pipe-networkconfiguration.html>`__
    """

    props: PropsDictType = {
        "AwsvpcConfiguration": (AwsVpcConfiguration, False),
    }


class PlacementConstraint(AWSProperty):
    """
    `PlacementConstraint <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pipes-pipe-placementconstraint.html>`__
    """

    props: PropsDictType = {
        "Expression": (str, False),
        "Type": (str, False),
    }


class PlacementStrategy(AWSProperty):
    """
    `PlacementStrategy <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pipes-pipe-placementstrategy.html>`__
    """

    props: PropsDictType = {
        "Field": (str, False),
        "Type": (str, False),
    }


class PipeTargetEcsTaskParameters(AWSProperty):
    """
    `PipeTargetEcsTaskParameters <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pipes-pipe-pipetargetecstaskparameters.html>`__
    """

    props: PropsDictType = {
        "CapacityProviderStrategy": ([CapacityProviderStrategyItem], False),
        "EnableECSManagedTags": (boolean, False),
        "EnableExecuteCommand": (boolean, False),
        "Group": (str, False),
        "LaunchType": (str, False),
        "NetworkConfiguration": (NetworkConfiguration, False),
        "Overrides": (EcsTaskOverride, False),
        "PlacementConstraints": ([PlacementConstraint], False),
        "PlacementStrategy": ([PlacementStrategy], False),
        "PlatformVersion": (str, False),
        "PropagateTags": (str, False),
        "ReferenceId": (str, False),
        "Tags": (Tags, False),
        "TaskCount": (integer, False),
        "TaskDefinitionArn": (str, True),
    }


class PipeTargetEventBridgeEventBusParameters(AWSProperty):
    """
    `PipeTargetEventBridgeEventBusParameters <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pipes-pipe-pipetargeteventbridgeeventbusparameters.html>`__
    """

    props: PropsDictType = {
        "DetailType": (str, False),
        "EndpointId": (str, False),
        "Resources": ([str], False),
        "Source": (str, False),
        "Time": (str, False),
    }


class PipeTargetHttpParameters(AWSProperty):
    """
    `PipeTargetHttpParameters <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pipes-pipe-pipetargethttpparameters.html>`__
    """

    props: PropsDictType = {
        "HeaderParameters": (dict, False),
        "PathParameterValues": ([str], False),
        "QueryStringParameters": (dict, False),
    }


class PipeTargetKinesisStreamParameters(AWSProperty):
    """
    `PipeTargetKinesisStreamParameters <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pipes-pipe-pipetargetkinesisstreamparameters.html>`__
    """

    props: PropsDictType = {
        "PartitionKey": (str, True),
    }


class PipeTargetLambdaFunctionParameters(AWSProperty):
    """
    `PipeTargetLambdaFunctionParameters <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pipes-pipe-pipetargetlambdafunctionparameters.html>`__
    """

    props: PropsDictType = {
        "InvocationType": (str, False),
    }


class PipeTargetRedshiftDataParameters(AWSProperty):
    """
    `PipeTargetRedshiftDataParameters <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pipes-pipe-pipetargetredshiftdataparameters.html>`__
    """

    props: PropsDictType = {
        "Database": (str, True),
        "DbUser": (str, False),
        "SecretManagerArn": (str, False),
        "Sqls": ([str], True),
        "StatementName": (str, False),
        "WithEvent": (boolean, False),
    }


class SageMakerPipelineParameter(AWSProperty):
    """
    `SageMakerPipelineParameter <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pipes-pipe-sagemakerpipelineparameter.html>`__
    """

    props: PropsDictType = {
        "Name": (str, True),
        "Value": (str, True),
    }


class PipeTargetSageMakerPipelineParameters(AWSProperty):
    """
    `PipeTargetSageMakerPipelineParameters <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pipes-pipe-pipetargetsagemakerpipelineparameters.html>`__
    """

    props: PropsDictType = {
        "PipelineParameterList": ([SageMakerPipelineParameter], False),
    }


class PipeTargetSqsQueueParameters(AWSProperty):
    """
    `PipeTargetSqsQueueParameters <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pipes-pipe-pipetargetsqsqueueparameters.html>`__
    """

    props: PropsDictType = {
        "MessageDeduplicationId": (str, False),
        "MessageGroupId": (str, False),
    }


class PipeTargetStateMachineParameters(AWSProperty):
    """
    `PipeTargetStateMachineParameters <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pipes-pipe-pipetargetstatemachineparameters.html>`__
    """

    props: PropsDictType = {
        "InvocationType": (str, False),
    }


class PipeTargetParameters(AWSProperty):
    """
    `PipeTargetParameters <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pipes-pipe-pipetargetparameters.html>`__
    """

    props: PropsDictType = {
        "BatchJobParameters": (PipeTargetBatchJobParameters, False),
        "CloudWatchLogsParameters": (PipeTargetCloudWatchLogsParameters, False),
        "EcsTaskParameters": (PipeTargetEcsTaskParameters, False),
        "EventBridgeEventBusParameters": (
            PipeTargetEventBridgeEventBusParameters,
            False,
        ),
        "HttpParameters": (PipeTargetHttpParameters, False),
        "InputTemplate": (str, False),
        "KinesisStreamParameters": (PipeTargetKinesisStreamParameters, False),
        "LambdaFunctionParameters": (PipeTargetLambdaFunctionParameters, False),
        "RedshiftDataParameters": (PipeTargetRedshiftDataParameters, False),
        "SageMakerPipelineParameters": (PipeTargetSageMakerPipelineParameters, False),
        "SqsQueueParameters": (PipeTargetSqsQueueParameters, False),
        "StepFunctionStateMachineParameters": (PipeTargetStateMachineParameters, False),
    }


class Pipe(AWSObject):
    """
    `Pipe <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-pipes-pipe.html>`__
    """

    resource_type = "AWS::Pipes::Pipe"

    props: PropsDictType = {
        "Description": (str, False),
        "DesiredState": (str, False),
        "Enrichment": (str, False),
        "EnrichmentParameters": (PipeEnrichmentParameters, False),
        "LogConfiguration": (PipeLogConfiguration, False),
        "Name": (str, False),
        "RoleArn": (str, True),
        "Source": (str, True),
        "SourceParameters": (PipeSourceParameters, False),
        "Tags": (dict, False),
        "Target": (str, True),
        "TargetParameters": (PipeTargetParameters, False),
    }



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.


import re

from .. import Join
from . import positive_integer

MINIMUM_MEMORY = 128
MAXIMUM_MEMORY = 10240


def validate_memory_size(memory_value):
    """Validate memory size for Lambda Function
    :param memory_value: The memory size specified in the Function
    :return: The provided memory size if it is valid
    Property: Function.MemorySize
    """

    memory_value = int(positive_integer(memory_value))
    if not MINIMUM_MEMORY <= memory_value <= MAXIMUM_MEMORY:
        raise ValueError(
            "Lambda Function memory size must be between %d and %d"
            % (MINIMUM_MEMORY, MAXIMUM_MEMORY)
        )
    return memory_value


def validate_package_type(package_type):
    """Validate PackageType for Lambda Function.
    :param package_type: The PackageType specified in the Function.
    :return: The provided package type if it is valid.
    Property: Function.PackageType
    """

    PACKAGE_TYPES = ["Image", "Zip"]

    if package_type not in PACKAGE_TYPES:
        raise ValueError(
            "Lambda Function PackageType must be one of: {}".format(
                ", ".join(PACKAGE_TYPES)
            )
        )
    return package_type


def validate_variables_name(variables):
    """
    Validate variable names for Lambda Function.
    Property: Environment.Variables
    """
    RESERVED_ENVIRONMENT_VARIABLES = [
        "AWS_ACCESS_KEY",
        "AWS_ACCESS_KEY_ID",
        "AWS_DEFAULT_REGION",
        "AWS_EXECUTION_ENV",
        "AWS_LAMBDA_FUNCTION_MEMORY_SIZE",
        "AWS_LAMBDA_FUNCTION_NAME",
        "AWS_LAMBDA_FUNCTION_VERSION",
        "AWS_LAMBDA_LOG_GROUP_NAME",
        "AWS_LAMBDA_LOG_STREAM_NAME",
        "AWS_REGION",
        "AWS_SECRET_ACCESS_KEY",
        "AWS_SECRET_KEY",
        "AWS_SECURITY_TOKEN",
        "AWS_SESSION_TOKEN",
        "LAMBDA_RUNTIME_DIR",
        "LAMBDA_TASK_ROOT",
        "TZ",
    ]
    ENVIRONMENT_VARIABLES_NAME_PATTERN = r"[a-zA-Z][a-zA-Z0-9_]+"

    for name in variables:
        if name in RESERVED_ENVIRONMENT_VARIABLES:
            raise ValueError(
                "Lambda Function environment variables names"
                " can't be none of:\n %s" % ", ".join(RESERVED_ENVIRONMENT_VARIABLES)
            )
        elif not re.match(ENVIRONMENT_VARIABLES_NAME_PATTERN, name):
            raise ValueError("Invalid environment variable name: %s" % name)

    return variables


def check_zip_file(zip_file):
    maxlength = 4 * 1024 * 1024  # 4MB
    toolong = (
        "ZipFile length cannot exceed %d characters. For larger "
        "source use S3Bucket/S3Key properties instead. "
        "Current length: %d"
    )

    if zip_file is None:
        return

    if isinstance(zip_file, str):
        z_length = len(zip_file)
        if z_length > maxlength:
            raise ValueError(toolong % (maxlength, z_length))
        return

    if isinstance(zip_file, Join):
        # This code tries to combine the length of all the strings in a
        # join. If a part is not a string, we do not count it (length 0).
        delimiter, values = zip_file.data["Fn::Join"]

        # Return if there are no values to join
        if not values or len(values) <= 0:
            return

        # Get the length of the delimiter
        if isinstance(delimiter, str):
            d_length = len(delimiter)
        else:
            d_length = 0

        # Get the length of each value that will be joined
        v_lengths = [len(v) for v in values if isinstance(v, str)]

        # Add all the lengths together
        z_length = sum(v_lengths)
        z_length += (len(values) - 1) * d_length

        if z_length > maxlength:
            raise ValueError(toolong % (maxlength, z_length))
        return


def validate_code(self):
    """
    Class: Code
    """
    image_uri = self.properties.get("ImageUri")
    zip_file = self.properties.get("ZipFile")
    s3_bucket = self.properties.get("S3Bucket")
    s3_key = self.properties.get("S3Key")
    s3_object_version = self.properties.get("S3ObjectVersion")

    if zip_file and image_uri:
        raise ValueError("You can't specify both 'ImageUri' and 'ZipFile'")
    if zip_file and s3_bucket:
        raise ValueError("You can't specify both 'S3Bucket' and 'ZipFile'")
    if zip_file and s3_key:
        raise ValueError("You can't specify both 'S3Key' and 'ZipFile'")
    if zip_file and s3_object_version:
        raise ValueError("You can't specify both 'S3ObjectVersion' and 'ZipFile'")
    if image_uri and (s3_bucket or s3_key or s3_object_version):
        raise ValueError(
            "You can't specify 'ImageUri' and any of 'S3Bucket', 'S3Key', "
            "or 'S3ObjectVersion' at the same time"
        )
    check_zip_file(zip_file)
    if not zip_file and not (s3_bucket and s3_key) and not image_uri:
        raise ValueError(
            "You must specify a bucket location (both the 'S3Bucket' and "
            "'S3Key' properties), the 'ImageUri' property, "
            "or the 'ZipFile' property"
        )


def validate_image_config(self):
    """
    Class: ImageConfig
    """

    command = self.properties.get("Command")
    if command and len(command) > 1500:
        raise ValueError("Maximum items in 'Command' is 1500")
    entry_point = self.properties.get("EntryPoint")
    if entry_point and len(entry_point) > 1500:
        raise ValueError("Maximum items in 'EntryPoint' is 1500")
    working_directory = self.properties.get("WorkingDirectory")
    if working_directory and len(working_directory) > 1000:
        raise ValueError("Maximum length of 'WorkingDirectory' is 1000")



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***


from . import AWSObject, AWSProperty, PropsDictType
from .validators import boolean, double


class ReferenceItem(AWSProperty):
    """
    `ReferenceItem <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-omics-variantstore-referenceitem.html>`__
    """

    props: PropsDictType = {
        "ReferenceArn": (str, True),
    }


class SseConfig(AWSProperty):
    """
    `SseConfig <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-omics-variantstore-sseconfig.html>`__
    """

    props: PropsDictType = {
        "KeyArn": (str, False),
        "Type": (str, True),
    }


class TsvStoreOptions(AWSProperty):
    """
    `TsvStoreOptions <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-omics-annotationstore-tsvstoreoptions.html>`__
    """

    props: PropsDictType = {
        "AnnotationType": (str, False),
        "FormatToHeader": (dict, False),
        "Schema": (dict, False),
    }


class StoreOptions(AWSProperty):
    """
    `StoreOptions <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-omics-annotationstore-storeoptions.html>`__
    """

    props: PropsDictType = {
        "TsvStoreOptions": (TsvStoreOptions, True),
    }


class AnnotationStore(AWSObject):
    """
    `AnnotationStore <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-omics-annotationstore.html>`__
    """

    resource_type = "AWS::Omics::AnnotationStore"

    props: PropsDictType = {
        "Description": (str, False),
        "Name": (str, True),
        "Reference": (ReferenceItem, False),
        "SseConfig": (SseConfig, False),
        "StoreFormat": (str, True),
        "StoreOptions": (StoreOptions, False),
        "Tags": (dict, False),
    }


class ReferenceStore(AWSObject):
    """
    `ReferenceStore <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-omics-referencestore.html>`__
    """

    resource_type = "AWS::Omics::ReferenceStore"

    props: PropsDictType = {
        "Description": (str, False),
        "Name": (str, True),
        "SseConfig": (SseConfig, False),
        "Tags": (dict, False),
    }


class RunGroup(AWSObject):
    """
    `RunGroup <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-omics-rungroup.html>`__
    """

    resource_type = "AWS::Omics::RunGroup"

    props: PropsDictType = {
        "MaxCpus": (double, False),
        "MaxDuration": (double, False),
        "MaxGpus": (double, False),
        "MaxRuns": (double, False),
        "Name": (str, False),
        "Tags": (dict, False),
    }


class SequenceStore(AWSObject):
    """
    `SequenceStore <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-omics-sequencestore.html>`__
    """

    resource_type = "AWS::Omics::SequenceStore"

    props: PropsDictType = {
        "Description": (str, False),
        "FallbackLocation": (str, False),
        "Name": (str, True),
        "SseConfig": (SseConfig, False),
        "Tags": (dict, False),
    }


class VariantStore(AWSObject):
    """
    `VariantStore <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-omics-variantstore.html>`__
    """

    resource_type = "AWS::Omics::VariantStore"

    props: PropsDictType = {
        "Description": (str, False),
        "Name": (str, True),
        "Reference": (ReferenceItem, True),
        "SseConfig": (SseConfig, False),
        "Tags": (dict, False),
    }


class WorkflowParameter(AWSProperty):
    """
    `WorkflowParameter <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-omics-workflow-workflowparameter.html>`__
    """

    props: PropsDictType = {
        "Description": (str, False),
        "Optional": (boolean, False),
    }


class Workflow(AWSObject):
    """
    `Workflow <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-omics-workflow.html>`__
    """

    resource_type = "AWS::Omics::Workflow"

    props: PropsDictType = {
        "Accelerators": (str, False),
        "DefinitionUri": (str, False),
        "Description": (str, False),
        "Engine": (str, False),
        "Main": (str, False),
        "Name": (str, False),
        "ParameterTemplate": (dict, False),
        "StorageCapacity": (double, False),
        "Tags": (dict, False),
    }
